@@ -12,7 +12,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-__version__ = '0.4.6'
+__version__ = "1.1.0"
+__all__ = [
+    "__version__",
+    "TemplateLoader",
+    "HTMLTemplate",
+    "TextTemplate",
+    "PigletError",
+    "PigletParseError",
+    "TemplateNotFound",
+]
 
 from piglet.loader import TemplateLoader
 from piglet.template import HTMLTemplate, TextTemplate
