@@ -28,7 +28,7 @@ from piglet.exceptions import PigletParseError
 @attr.s
 class Statement(parse.ParseItem):
 
-    EMPTY_TAGS = 'import'
+    EMPTY_TAGS = {"import", "include"}
 
     name = attr.ib(default=None)
     args = attr.ib(default=None)
@@ -42,21 +42,22 @@ class Statement(parse.ParseItem):
 
     @property
     def source(self):
-        return '{{%{0.space1}{0.name}{0.space2}{0.args}%}}'.format(self)
+        return "{{%{0.space1}{0.name}{0.space2}{0.args}%}}".format(self)
 
     @property
     def raw_args(self):
-        return ''.join(a.source if isinstance(a, QuotedString) else a
-                       for a in self.args)
+        return "".join(
+            a.source if isinstance(a, QuotedString) else a for a in self.args
+        )
 
     def is_open_tag(self):
-        return not self.name.startswith('end')
+        return not self.name.startswith("end")
 
     def is_openclose_tag(self):
         return self.name in self.EMPTY_TAGS
 
     def is_close_tag(self):
-        return self.name.startswith('end')
+        return self.name.startswith("end")
 
 
 @attr.s
@@ -66,7 +67,7 @@ class QuotedString(parse.ParseItem):
 
     @property
     def source(self):
-        return '{0.quote}{0.content}{0.quote}'.format(self)
+        return "{0.quote}{0.content}{0.quote}".format(self)
 
 
 @attr.s
@@ -74,11 +75,12 @@ class Container(object):
     children = attr.ib(default=attr.Factory(list))
 
 
-parser_ns = {'Statement': Statement,
-             'Text': parse.Text,
-             'QuotedString': QuotedString,
-             }
-tt_grammar = resource_string(__name__, 'grammar_tt.txt').decode('UTF-8')
+parser_ns = {
+    "Statement": Statement,
+    "Text": parse.Text,
+    "QuotedString": QuotedString,
+}
+tt_grammar = resource_string(__name__, "grammar_tt.txt").decode("UTF-8")
 tt_parser = parsley.makeGrammar(tt_grammar, parser_ns)
 
 
@@ -108,14 +110,15 @@ def make_tree(parse_result):
         elif isinstance(item, Statement) and item.is_close_tag():
             # else statements don't have associated end tags - the end tag
             # should cause the parent if statement to close instead
-            if head.name == 'else':
+            if head.name == "else":
                 head = stack.pop()
             name = item.name[3:]
             if name and name != head.name:
-                raise parse.PigletParseError("Unexpected {} at {}:{}"
-                                             .format(item.source,
-                                                     item.pos.line,
-                                                     item.pos.char))
+                raise parse.PigletParseError(
+                    "Unexpected {} at {}:{}".format(
+                        item.source, item.pos.line, item.pos.char
+                    )
+                )
             head.end_tag = item
             head.end_tag_pos = item.pos
             stack.pop()
@@ -123,20 +126,18 @@ def make_tree(parse_result):
         else:
             head.children.append(item)
     if len(stack) > 1:
-        raise parse.PigletParseError("Missing end tag for {}, "
-                                     "opened at {}:{}"
-                                     .format(stack[-1].source,
-                                             stack[-1].pos.line,
-                                             stack[-1].pos.char))
+        raise parse.PigletParseError(
+            "Missing end tag for {}, "
+            "opened at {}:{}".format(
+                stack[-1].source, stack[-1].pos.line, stack[-1].pos.char
+            )
+        )
     return stack[0]
 
 
 def compile_intermediate(parse_result):
     tree = make_tree(parse_result)
     strip_ws_on_directive_lines(tree)
-    cx.strip_spaces_between_directives(tree,
-                                       element_cls=Statement,
-                                       is_directive=lambda x: isinstance(x, Statement))
     compiled = _compile_node(tree)
     cx._strip_unwanted_nodes(compiled)
     cx._concatentate_adjacent_text_nodes(compiled)
@@ -147,25 +148,21 @@ def compile_intermediate(parse_result):
 def strip_ws_on_directive_lines(node):
 
     # Remove whitespace until after first newline
-    trim_line_start = partial(re.compile(r'\A[ \t]*(\r\n|\n|\r)').sub, r'')
+    trim_line_start = partial(re.compile(r"\A[ \t]*(\r\n|\n|\r)").sub, r"")
 
     # Remove whitespace after last newline
-    trim_line_end = partial(re.compile(r'([\r\n])[ \t]*\Z').sub, r'\1')
+    trim_line_end = partial(re.compile(r"([\r\n])[ \t]*\Z").sub, r"\1")
 
     # Remove leading whitespace
-    trim_all_start = partial(re.compile(r'\A\s*', re.S).sub, r'')
+    trim_all_start = partial(re.compile(r"\A\s*", re.S).sub, r"")
 
     # Remove trailing whitespace
-    trim_all_end = partial(re.compile(r'\s*\Z', re.S).sub, r'')
+    trim_all_end = partial(re.compile(r"\s*\Z", re.S).sub, r"")
 
-    trim_nothing = lambda s: s
+    trim_nothing = lambda s: s  # noqa
 
-    trim_start = {'+': trim_nothing,
-                  '-': trim_all_start,
-                   None: trim_line_start}
-    trim_end = {'+': trim_nothing,
-                '-': trim_all_end,
-                None: trim_line_end}
+    trim_start = {"+": trim_nothing, "-": trim_all_start, None: trim_line_start}
+    trim_end = {"+": trim_nothing, "-": trim_all_end, None: trim_line_end}
 
     if isinstance(node, Statement):
         if node.children:
@@ -190,31 +187,49 @@ def strip_ws_on_directive_lines(node):
             n.content = trim_start[p.end_tag.ws_after](n.content)
 
     for item in node.children:
-        if hasattr(item, 'children'):
+        if hasattr(item, "children"):
             strip_ws_on_directive_lines(item)
 
 
-def compile_extends(stmt, href):
-    return im.ExtendsNode(href=href,
-                          pos=stmt.pos,
-                          children=[_compile_node(c) for c in stmt.children])
+def compile_include(stmt, href, *args):
+    ignore_missing = "ignore-missing" in args
+    return im.IncludeNode(
+        href=href,
+        pos=stmt.pos,
+        ignore_missing=ignore_missing,
+    )
+
+
+def compile_extends(stmt, href, *args):
+    ignore_missing = "ignore-missing" in args
+    return im.ExtendsNode(
+        href=href,
+        pos=stmt.pos,
+        children=[_compile_node(c) for c in stmt.children],
+        ignore_missing=ignore_missing,
+    )
 
 
 def compile_block(stmt, name):
-    return im.BlockNode(name=name,
-                        pos=stmt.pos,
-                        children=[_compile_node(c) for c in stmt.children])
+    return im.BlockNode(
+        name=name, pos=stmt.pos, children=[_compile_node(c) for c in stmt.children]
+    )
 
 
 def compile_for(stmt, *args):
-    return im.ForNode(each=stmt.raw_args,
-                      pos=stmt.pos,
-                      children=[_compile_node(c) for c in stmt.children])
+    return im.ForNode(
+        each=stmt.raw_args,
+        pos=stmt.pos,
+        children=[_compile_node(c) for c in stmt.children],
+    )
 
 
 def compile_def(stmt, *args):
-    return im.DefNode(function=stmt.raw_args, pos=stmt.pos,
-                      children=[_compile_node(c) for c in stmt.children])
+    return im.DefNode(
+        function=stmt.raw_args,
+        pos=stmt.pos,
+        children=[_compile_node(c) for c in stmt.children],
+    )
 
 
 def compile_if(stmt, *args):
@@ -223,54 +238,58 @@ def compile_if(stmt, *args):
     children = stmt.children
     if children:
         last = children[-1]
-        if isinstance(last, Statement) and last.name == 'else':
+        if isinstance(last, Statement) and last.name == "else":
             children = children[:-1]
             n.else_ = im.ElseNode(
-                pos=last.pos,
-                children=[_compile_node(c) for c in last.children])
+                pos=last.pos, children=[_compile_node(c) for c in last.children]
+            )
     n.children = [_compile_node(c) for c in children]
     return n
 
 
 def compile_with(stmt, *args):
-    return im.WithNode(vars=stmt.raw_args,
-                       children=[_compile_node(c) for c in stmt.children],
-                       pos=stmt.pos)
+    return im.WithNode(
+        vars=stmt.raw_args,
+        children=[_compile_node(c) for c in stmt.children],
+        pos=stmt.pos,
+    )
 
 
 def compile_trans(stmt, message=None):
     return im.TranslationNode(
         message=message,
         children=[_compile_node(c) for c in stmt.children],
-        pos=stmt.pos)
+        pos=stmt.pos,
+    )
 
 
 def compile_transname(stmt, name):
     return im.TranslationPlaceholder(
-        name=name,
-        children=[_compile_node(c) for c in stmt.children],
-        pos=stmt.pos)
+        name=name, children=[_compile_node(c) for c in stmt.children], pos=stmt.pos
+    )
 
 
 def compile_import(stmt, href, as_, alias, *args):
-    if as_ != 'as' or args:
+    if as_ != "as" or args:
         raise PigletParseError(
             "Syntax error: "
-            "should be \"import 'path/to/template.txt' as alias_name\".")
+            "should be \"import 'path/to/template.txt' as alias_name\"."
+        )
 
     return im.ImportNode.factory(href=href, alias=alias, pos=stmt.pos)
 
 
 DIRECTIVES = {
-    'extends': compile_extends,
-    'block': compile_block,
-    'for': compile_for,
-    'if': compile_if,
-    'def': compile_def,
-    'with': compile_with,
-    'trans': compile_trans,
-    'transname': compile_transname,
-    'import': compile_import,
+    "include": compile_include,
+    "extends": compile_extends,
+    "block": compile_block,
+    "for": compile_for,
+    "if": compile_if,
+    "def": compile_def,
+    "with": compile_with,
+    "trans": compile_trans,
+    "transname": compile_transname,
+    "import": compile_import,
 }
 
 
