@@ -16,42 +16,44 @@
 Compile XML to an intermediate parse tree
 """
 from copy import copy
-from itertools import chain, product, takewhile
+from itertools import chain
+from itertools import product
+from itertools import takewhile
 import textwrap
 import re
 
 import attr
 
-from piglet import parse, intermediate as im, interpolate
+from piglet import intermediate as im
+from piglet import interpolate
+from piglet import parse
 from piglet.exceptions import PigletParseError
 
-XMLNS = {'py': 'py',
-         'i18n': 'i18n'}
+XMLNS = {"py": "py", "i18n": "i18n"}
 
 # Patterns to match whitespace (but not non-breaking spaces or other unicode
 # whitespace)
-ENTIRELY_WS = re.compile(r'^[ \t\r\n]*$')
-SURROUNDING_WS = re.compile(r'^'
-                            r'([ \t\r\n]*)'
-                            r'(.*[^ \t\r\n])'
-                            r'([ \t\r\n]*)$')
-WS_L = re.compile(r'^([ \t\r\n]*)(.*[^ \t\r\n].*)?$', re.S)
-WS_R = re.compile(r'(^|(?:.*[^ \t\r\n]))([ \t\r\n]*)$')
-CRNL = re.compile(r'[\r\n]')
+ENTIRELY_WS = re.compile(r"^[ \t\r\n]*$")
+SURROUNDING_WS = re.compile(r"^" r"([ \t\r\n]*)" r"(.*[^ \t\r\n])" r"([ \t\r\n]*)$")
+WS_L = re.compile(r"^([ \t\r\n]*)(.*[^ \t\r\n].*)?$", re.S)
+WS_R = re.compile(r"(^|(?:.*[^ \t\r\n]))([ \t\r\n]*)$")
+CRNL = re.compile(r"[\r\n]")
 
 #: List of attributes that should be omitted if the interpolated value
 #: evaluates to None or False.
 #: For example '<input checked="$checked"/>' should be rendered as '<input/>'
 #: in the case that checked == False.
-HTML_EMPTY_ATTRS = {'checked',
-                    'declare',
-                    'defer',
-                    'disabled',
-                    'ismap',
-                    'multiple',
-                    'nohref',
-                    'readonly',
-                    'selected'}
+HTML_EMPTY_ATTRS = {
+    "checked",
+    "declare",
+    "defer",
+    "disabled",
+    "ismap",
+    "multiple",
+    "nohref",
+    "readonly",
+    "selected",
+}
 
 try:
     from html import unescape
@@ -67,6 +69,7 @@ except ImportError:
 
         Adapted from http://effbot.org/zone/re-sub.htm#unescape-html
         """
+
         def fixup(m):
             text = m.group(0)
             if text[:2] == "&#":
@@ -83,11 +86,11 @@ except ImportError:
                 except KeyError:
                     pass
             return text
-        return re.sub("&#?\w+;", fixup, text)
 
-ustr = type(u'')
+        return re.sub(r"&#?\w+;", fixup, text)
 
-WHITESPACE = 'py:whitespace'
+
+WHITESPACE = "py:whitespace"
 
 
 class ElsePlaceholder(im.DirectiveNode, im.ContainerNode):
@@ -123,7 +126,7 @@ class Directive(object):
     extra_attrs = attr.ib(default=attr.Factory(list))
 
     inner = attr.ib(default=False)
-    ns = attr.ib(default=XMLNS['py'])
+    ns = attr.ib(default=XMLNS["py"])
     aliases = attr.ib(default=tuple())
 
     attr = attr_
@@ -131,7 +134,7 @@ class Directive(object):
 
     @property
     def qname(self):
-        return '{}:{}'.format(self.ns, self.name)
+        return "{}:{}".format(self.ns, self.name)
 
     def get_extra_attrs(self, data, el):
         """
@@ -145,12 +148,18 @@ class Directive(object):
         """
         return None
 
+    def get_tagname_expr(self, data, el):
+        """
+        Return a python expression for the tag name, or None
+        """
+        return None
+
     def q(self, n):
         """
         Qualify name ``n`` by putting it in the directive's configured
         namespace
         """
-        return '{}:{}'.format(self.ns, n)
+        return "{}:{}".format(self.ns, n)
 
     def parse(self, el):
         """
@@ -179,18 +188,25 @@ class Directive(object):
         data = {}
         pos = el.pos
         if el.qname == tagname:
-            remaining = Fragment(children=el.children,
-                                 pos=el.pos,
-                                 attrs=el.attrs)
+            remaining = Fragment(
+                children=el.children,
+                pos=el.pos,
+                end=el.end,
+                close_tag_pos=el.close_tag_pos,
+                attrs=el.attrs,
+            )
+            remaining.close_tag = el.close_tag
             if not self.empty:
                 try:
                     data = {self.attr: remaining.attrs.pop(self.q(self.attr)).value}
                 except KeyError:
                     if self.default is self.REQUIRED:
                         raise PigletParseError(
-                            "Missing attribute {!r} in element {}"
-                            .format(self.q(self.attr), el),
-                            lineno=el.pos.line)
+                            "Missing attribute {!r} in element {}".format(
+                                self.q(self.attr), el
+                            ),
+                            lineno=el.pos.line,
+                        )
                     else:
                         data = {self.attr: self.default}
 
@@ -212,27 +228,32 @@ class Directive(object):
             except KeyError:
                 if required:
                     raise PigletParseError(
-                        "Missing attribute {!r} in element {}"
-                        .format(self.q(extra), el),
-                        lineno=el.pos.line)
+                        "Missing attribute {!r} in element {}".format(
+                            self.q(extra), el
+                        ),
+                        lineno=el.pos.line,
+                    )
 
         data = {k: unescape(v) for k, v in data.items()}
         return (data, remaining, pos)
 
 
 class PyAttrsDirective(Directive):
-
     def get_extra_attrs(self, data, element):
-        return data['attrs']
+        return data["attrs"]
 
 
 class PyStripDirective(Directive):
-
     def get_strip_condition(self, data, el):
-        if data['strip'].strip() == '':
-            return 'True'
+        if data["strip"].strip() == "":
+            return "True"
         else:
-            return data['strip']
+            return data["strip"]
+
+
+class PyTagDirective(Directive):
+    def get_tagname_expr(self, data, el):
+        return data["tag"]
 
 
 def make_i18n_substitution(name, expr=None):
@@ -252,65 +273,103 @@ def make_i18n_from_comment(comment):
 
 
 directives = [
-    Directive(name='comment',
-              attr='comment',
-              empty=True,
-              factory=im.Comment.factory),
-    Directive(name='with', attr='vars', factory=im.WithNode.factory),
-    Directive(name='def', attr='function', factory=im.DefNode.factory),
-    Directive(name='extends', attr='href', factory=im.ExtendsNode.factory),
-    Directive(name='include', attr='href', factory=im.IncludeNode.factory),
-    Directive(name='block', attr='name', inner=True, factory=im.BlockNode.factory),
-    Directive(name='import', attr='href', extra_attrs=[('alias', True)],
-              factory=im.ImportNode.factory),
-    Directive(name='when', attr='test', factory=im.WhenNode.factory),
-    Directive(name='case', attr='value',
-              factory=lambda value: im.WhenNode.factory(test=value)),
-    Directive(name='otherwise', attr='otherwise', empty=True,
-              factory=im.OtherwiseNode.factory),
-    Directive(name='else', attr='else', empty=True, factory=ElsePlaceholder),
-    Directive(name='for', attr='each', factory=im.ForNode.factory),
-    Directive(name='if', attr='test', factory=im.IfNode.factory),
-    Directive(name='switch', attr='test', default='True',
-              factory=im.ChooseNode.factory),
-    Directive(name='choose', attr='test', default='True',
-              factory=im.ChooseNode.factory),
-    Directive(name='call', attr='function', inner=True, factory=im.Call.factory),
-    Directive(name='keyword', attr='name', factory=im.CallKeyword.factory),
-    Directive(name='content',
-              attr='value',
-              inner=True,
-              factory=im.InterpolateNode.factory),
-    Directive(name='replace', attr='value', factory=im.InterpolateNode.factory),
-    PyAttrsDirective(name=(None, 'attrs'), attr='attrs', factory=None),
-    PyStripDirective(name=(None, 'strip'), attr='strip', factory=None),
-    Directive(ns=XMLNS['i18n'],
-              inner=True,
-              name='translate',
-              aliases=['trans', 'message'],
-              attr='message',
-              default='',
-              extra_attrs=[('comment', False)],
-              factory=im.TranslationNode),
-    Directive(ns=XMLNS['i18n'],
-              name='name',
-              attr='name',
-              default=None,
-              aliases=['s'],
-              extra_attrs=[('expr', False)],
-              factory=make_i18n_substitution),
-    Directive(ns=XMLNS['i18n'],
-              inner=True,
-              name='comment',
-              attr='comment',
-              factory=make_i18n_from_comment),
+    Directive(name="comment", attr="comment", empty=True, factory=im.Comment.factory),
+    Directive(name="with", attr="vars", factory=im.WithNode.factory),
+    Directive(name="def", attr="function", factory=im.DefNode.factory),
+    Directive(
+        name="extends",
+        attr="href",
+        extra_attrs=[("ignore-missing", False)],
+        factory=im.ExtendsNode.factory,
+    ),
+    Directive(name="include", attr="href", factory=im.IncludeNode.factory),
+    Directive(name="block", attr="name", inner=True, factory=im.BlockNode.factory),
+    Directive(
+        name="import",
+        attr="href",
+        extra_attrs=[("alias", True), ("ignore-missing", False)],
+        factory=im.ImportNode.factory,
+    ),
+    Directive(name="when", attr="test", factory=im.WhenNode.factory),
+    Directive(
+        name="case",
+        attr="value",
+        factory=lambda value: im.WhenNode.factory(test=value),
+    ),
+    Directive(
+        name="otherwise",
+        attr="otherwise",
+        empty=True,
+        factory=im.OtherwiseNode.factory,
+    ),
+    Directive(name="else", attr="else", empty=True, factory=ElsePlaceholder),
+    Directive(name="for", attr="each", factory=im.ForNode.factory),
+    Directive(name="if", attr="test", factory=im.IfNode.factory),
+    Directive(
+        name="switch",
+        attr="test",
+        default="True",
+        factory=im.ChooseNode.factory,
+    ),
+    Directive(
+        name="choose",
+        attr="test",
+        default="True",
+        factory=im.ChooseNode.factory,
+    ),
+    Directive(name="call", attr="function", inner=True, factory=im.Call.factory),
+    Directive(name="keyword", attr="name", factory=im.CallKeyword.factory),
+    Directive(
+        name="content",
+        attr="value",
+        inner=True,
+        factory=im.InterpolateNode.factory,
+    ),
+    Directive(name="replace", attr="value", factory=im.InterpolateNode.factory),
+    Directive(name="filter", attr="function", inner=True, factory=im.FilterNode),
+    PyAttrsDirective(name=(None, "attrs"), attr="attrs", factory=None),
+    PyStripDirective(name=(None, "strip"), attr="strip", factory=None),
+    PyTagDirective(name="tag", attr="tag", factory=None),
+    Directive(
+        ns=XMLNS["i18n"],
+        inner=True,
+        name="translate",
+        aliases=["trans", "message"],
+        attr="message",
+        default="",
+        extra_attrs=[("comment", False)],
+        factory=im.TranslationNode,
+    ),
+    Directive(
+        ns=XMLNS["i18n"],
+        name="name",
+        attr="name",
+        default=None,
+        aliases=["s"],
+        extra_attrs=[("expr", False)],
+        factory=make_i18n_substitution,
+    ),
+    Directive(
+        ns=XMLNS["i18n"],
+        inner=True,
+        name="comment",
+        attr="comment",
+        factory=make_i18n_from_comment,
+    ),
 ]
 
 
 class Element(object):
-
-    def __init__(self, qname='', space='', attrs='', pos=None, end='',
-                 close_tag_pos=None, children=None):
+    def __init__(
+        self,
+        qname="",
+        space="",
+        attrs="",
+        pos=None,
+        end="",
+        close_tag_pos=None,
+        children=None,
+    ):
         self.qname = qname
         self.space = space
         self.attrs = attrs
@@ -322,21 +381,22 @@ class Element(object):
         attrs = self.formatted_attrs()
         if pos is not None:
             self.end_pos = self.pos.advance(
-                '<{}{}{}'.format(self.qname, self.space, attrs))
-        if ':' in qname:
-            self.ns = qname.split(':')[0]
+                "<{}{}{}".format(self.qname, self.space, attrs)
+            )
+        if ":" in qname:
+            self.ns = qname.split(":")[0]
         else:
-            self.ns = 'html'
+            self.ns = "html"
 
         for name in list(self.attrs):
             savedname = name
-            if ':' not in name:
-                name = '{}:{}'.format(self.ns, name)
+            if ":" not in name:
+                name = "{}:{}".format(self.ns, name)
             self.attrs[name] = self.attrs.pop(savedname)
 
     def __str__(self):
         if self.close_tag:
-            return self.open_tag() + '...' + self.close_tag
+            return self.open_tag() + "..." + self.close_tag
         else:
             return self.open_tag()
 
@@ -353,29 +413,28 @@ class Element(object):
         self.children.append(node)
 
     def is_dynamic(self):
-        namespaces = ['{}:'.format(ns) for ns in XMLNS]
+        namespaces = ["{}:".format(ns) for ns in XMLNS]
         names = chain([self.qname], self.attrs)
         return any(x.startswith(ns) for x, ns in product(names, namespaces))
 
     def open_tag(self):
         attrs = self.formatted_attrs()
-        return '<{}{}{}{}'.format(self.qname, self.space, attrs, self.end)
+        return "<{}{}{}{}".format(self.qname, self.space, attrs, self.end)
 
     def formatted_attrs(self):
         if self.attrs:
-            return ''.join(v.source for v in self.attrs.values())
-        return ''
+            return "".join(v.source for v in self.attrs.values())
+        return ""
 
 
 class Fragment(Element):
-
     def __init__(self, *args, **kwargs):
-        if 'qname' not in kwargs:
-            kwargs['qname'] = '#fragment'
+        if "qname" not in kwargs:
+            kwargs["qname"] = "#fragment"
         super(Fragment, self).__init__(*args, **kwargs)
 
     def open_tag(self):
-        return ''
+        return ""
 
 
 def get_directives(element):
@@ -397,13 +456,14 @@ def get_directives(element):
             result.append((d, data, pos))
 
     if not remaining.attrs:
-        remaining.space = ''
+        remaining.space = ""
 
     for name in chain([remaining.qname], remaining.attrs):
-        if name.startswith('py:'):
-            raise PigletParseError('Unrecognized directive {!r} in element {}'
-                                   .format(name, element),
-                                   lineno=element.pos.line)
+        if name.startswith("py:"):
+            raise PigletParseError(
+                "Unrecognized directive {!r} in element {}".format(name, element),
+                lineno=element.pos.line,
+            )
     return result, remaining
 
 
@@ -416,29 +476,36 @@ def make_dom(parse_result):
     head = stack[-1]
     for item in parse_result:
         if isinstance(item, parse.OpenCloseTag):
-            head.append(Element(qname=item.qname,
-                                space=item.space,
-                                attrs=item.attrs,
-                                pos=item.pos,
-                                end='/>',
-                                close_tag_pos=item.pos))
+            head.append(
+                Element(
+                    qname=item.qname,
+                    space=item.space,
+                    attrs=item.attrs,
+                    pos=item.pos,
+                    end="/>",
+                    close_tag_pos=item.pos,
+                )
+            )
 
         elif isinstance(item, parse.OpenTag):
-            newhead = Element(qname=item.qname,
-                              space=item.space,
-                              attrs=item.attrs,
-                              pos=item.pos,
-                              end='>')
+            newhead = Element(
+                qname=item.qname,
+                space=item.space,
+                attrs=item.attrs,
+                pos=item.pos,
+                end=">",
+            )
             head.append(newhead)
             stack.append(newhead)
             head = newhead
 
         elif isinstance(item, parse.CloseTag):
             if item.qname != head.qname:
-                raise parse.PigletParseError("Unexpected {} at {}:{}"
-                                             .format(item.source,
-                                                     item.pos.line,
-                                                     item.pos.char))
+                raise parse.PigletParseError(
+                    "Unexpected {} at {}:{}".format(
+                        item.source, item.pos.line, item.pos.char
+                    )
+                )
             head.close_tag = item.source
             head.close_tag_pos = item.pos
             stack.pop()
@@ -447,23 +514,23 @@ def make_dom(parse_result):
         else:
             head.append(item)
     if len(stack) > 1:
-        raise parse.PigletParseError("Missing closing tag for <{}>, "
-                                     "opened at {}:{}"
-                                     .format(stack[-1].qname,
-                                             stack[-1].pos.line,
-                                             stack[-1].pos.char))
+        raise parse.PigletParseError(
+            "Missing closing tag for <{}>, "
+            "opened at {}:{}".format(
+                stack[-1].qname, stack[-1].pos.line, stack[-1].pos.char
+            )
+        )
     return stack[0]
 
 
-def strip_spaces_between_directives(node,
-                                    element_cls=Element,
-                                    is_directive=(
-                                        lambda n: isinstance(n, Element) and
-                                        n.qname.startswith('py:'))
-                                    ):
+def strip_spaces_between_directives(
+    node,
+    element_cls=Element,
+    is_directive=(lambda n: isinstance(n, Element) and n.qname.startswith("py:")),
+):
     st = None
     ws = []
-    FOUND_DIRECTIVE = ['FOUND']
+    FOUND_DIRECTIVE = ["FOUND"]
 
     for n in node.children:
         is_ws = isinstance(n, parse.Text) and ENTIRELY_WS.match(n.content)
@@ -489,21 +556,21 @@ def strip_spaces_between_directives(node,
 
 def strip_whitespace(node, strip):
 
-    stripvalues = {'strip': True, 'preserve': False}
+    stripvalues = {"strip": True, "preserve": False}
 
     def is_whitespace(n):
         return isinstance(n, parse.Text) and ENTIRELY_WS.match(n.content)
 
     if isinstance(node, Element):
         if node.qname == WHITESPACE:
-            strip = stripvalues[node.attrs['py:value'].value]
+            strip = stripvalues[node.attrs["py:value"].value]
             node.__class__ = Fragment
 
         elif WHITESPACE in node.attrs:
             strip = stripvalues[node.attrs[WHITESPACE].value]
             del node.attrs[WHITESPACE]
             if len(node.attrs) == 0:
-                node.space = ''
+                node.space = ""
 
     if strip:
         newchildren = []
@@ -525,13 +592,13 @@ def strip_whitespace(node, strip):
             if preceding is None:
                 mo = WS_L.match(n.content)
                 if mo:
-                    n.content = mo.group(2) or ''
+                    n.content = mo.group(2) or ""
                     n.pos = n.pos.advance(mo.group(1))
 
             if following is None:
                 mo = WS_R.match(n.content)
                 if mo:
-                    n.content = mo.group(1) or ''
+                    n.content = mo.group(1) or ""
 
             if isinstance(following, Element):
                 mo = WS_R.match(n.content)
@@ -565,7 +632,7 @@ def combine_entities(node):
     newchildren = []
 
     for n in node.children:
-        if hasattr(n, 'children'):
+        if hasattr(n, "children"):
             combine_entities(n)
         if isinstance(n, parse.Text):
             if acc is None:
@@ -605,13 +672,13 @@ def _strip_unwanted_nodes(root):
         if not parent:
             continue
         if node.__class__ is im.ContainerNode:
-            parent.children[pos:pos + 1] = node.children
+            parent.children[pos : pos + 1] = node.children
 
-        elif node.__class__ is im.TextNode and node.content == '':
-            parent.children[pos:pos + 1] = []
+        elif node.__class__ is im.TextNode and node.content == "":
+            parent.children[pos : pos + 1] = []
 
         elif node.__class__ is im.NullNode:
-            parent.children[pos:pos + 1] = []
+            parent.children[pos : pos + 1] = []
 
 
 def _concatentate_adjacent_text_nodes(root):
@@ -623,8 +690,8 @@ def _concatentate_adjacent_text_nodes(root):
         except IndexError:
             continue
         if isinstance(sibling, im.TextNode):
-            node.content = ''.join([node.content, sibling.content])
-            parent.children[pos:pos + 2] = [node]
+            node.content = "".join([node.content, sibling.content])
+            parent.children[pos : pos + 2] = [node]
 
 
 def _postprocess(root):
@@ -644,8 +711,7 @@ def pp_move_else_inside_if(elsenode, parent):
             item.else_ = im.ElseNode(children=elsenode.children)
             break
     else:
-        raise PigletParseError('py:else without an py:if',
-                               lineno=elsenode.pos.line)
+        raise PigletParseError("py:else without an py:if", lineno=elsenode.pos.line)
 
     parent.children.remove(elsenode)
 
@@ -666,16 +732,14 @@ def _compile_node(node):
     elif isinstance(node, Element):
         directives, remaining = get_directives(node)
         if directives:
-            return _compile_element_with_directives(
-                node, directives, remaining)
+            return _compile_element_with_directives(node, directives, remaining)
 
         else:
             return _compile_vanilla_element(node)
 
     elif isinstance(node, parse.PI):
-        if node.target == 'python':
-            return im.InlineCodeNode(pysrc=textwrap.dedent(node.content),
-                                     pos=node.pos)
+        if node.target == "python":
+            return im.InlineCodeNode(pysrc=textwrap.dedent(node.content), pos=node.pos)
         else:
             return im.TextNode.factory(node.source, pos=node.pos)
 
@@ -683,10 +747,10 @@ def _compile_node(node):
         return im.TextNode.factory(node.source, pos=node.pos)
 
     elif isinstance(node, parse.Comment):
-        c = '<!--' + node.content + '-->'
+        c = "<!--" + node.content + "-->"
         n = im.TextNode.factory(c, pos=node.pos)
-        if node.content.startswith('!'):
-            n.content = ''
+        if node.content.startswith("!"):
+            n.content = ""
         return n
 
     elif isinstance(node, parse.Declaration):
@@ -710,21 +774,22 @@ def _compile_text(node):
     container = im.ContainerNode()
     pos = node.pos
     for i in items:
-        if isinstance(i, (str, ustr)):
+        if isinstance(i, str):
             container.append(im.TextNode(i, pos=pos))
             pos = pos.advance(i)
         elif isinstance(i, interpolate.Interpolation):
-            container.append(im.InterpolateNode.factory(
-                value=unescape(i.value),
-                pos=pos,
-                autoescape=i.autoescape and (not node.cdata)))
+            container.append(
+                im.InterpolateNode.factory(
+                    value=unescape(i.value),
+                    pos=pos,
+                    autoescape=i.autoescape and (not node.cdata),
+                )
+            )
             pos = pos.advance(i.source)
     return container
 
 
-def _compile_open_tag(node,
-                      extra_attrs=None,
-                      strip_condition=None):
+def _compile_open_tag(node, tagname_expr=None, extra_attrs=None, strip_condition=None):
     """
     Compile the open tag attributes.
     This needs special casing when interpolations are present, eg in the case:
@@ -735,58 +800,69 @@ def _compile_open_tag(node,
     evaluates to None.
 
     :param node: the parse.Element node to compile
+    :param tagname_expr: a python expression to replace the tag name
     :param extra_attrs: A list of python source expressions, each returning a
                         list of attributes
     :param strip_condition: a python boolean expression to decide whether
                             to output the open tag at all
     """
-
-    if isinstance(node, Fragment):
+    if isinstance(node, Fragment) and not tagname_expr:
         return im.ContainerNode(pos=node.pos)
 
     container = wn = im.ContainerNode(pos=node.pos)
+
     if strip_condition:
-        if strip_condition in {'True', '1'}:
+        if strip_condition in {"True", "1"}:
             container.append(im.NullNode(pos=node.pos))
         else:
             container.append(
-                im.IfNode(test='not ({})'.format(strip_condition),
-                          children=[],
-                          pos=node.pos)
+                im.IfNode(
+                    test="not ({})".format(strip_condition), children=[], pos=node.pos
+                )
             )
         wn = container.tip()
 
-    wn.append(im.TextNode.factory('<{}{}'.format(node.qname, node.space), pos=node.pos))
+    wn.append(im.TextNode.factory("<", pos=node.pos))
+    if tagname_expr:
+        wn.append(im.InterpolateNode(value=tagname_expr, pos=node.end_pos))
+    else:
+        wn.append(im.TextNode.factory(node.qname, pos=node.pos))
+    wn.append(im.TextNode.factory(node.space, pos=node.pos))
     if extra_attrs:
-        wn.append(im.InlineCodeNode(pysrc='__piglet_attrs = {}', pos=node.pos))
+        wn.append(im.InlineCodeNode(pysrc="__piglet_attrs = {}", pos=node.pos))
         for item in extra_attrs:
-            wn.append(im.InlineCodeNode(pysrc='__piglet_attrs.update({})'
-                                        .format(item),
-                                        pos=node.pos))
+            wn.append(
+                im.InlineCodeNode(
+                    pysrc="__piglet_attrs.update({})".format(item), pos=node.pos
+                )
+            )
 
     for a in node.attrs.values():
         if node.attrs and extra_attrs:
-            wn.append(im.IfNode(
-                test="'{}' not in __piglet_attrs".format(a.name),
-                pos=a.pos))
+            wn.append(
+                im.IfNode(test="'{}' not in __piglet_attrs".format(a.name), pos=a.pos)
+            )
             _compile_attr(a, wn.tip())
         else:
             _compile_attr(a, wn)
 
     if extra_attrs:
-        for_ = im.ForNode(each=('__piglet_attr_k, __piglet_attr_v '
-                                'in __piglet_attrs.items()'),
-                          pos=node.end_pos,
-                          children=[])
-        if_ = im.IfNode(test='__piglet_attr_v is not None',
-                        pos=node.end_pos,
-                        children=[im.TextNode(' ', pos=node.end_pos),
-                                  im.InterpolateNode(
-                                      value='__piglet_attr_k', pos=node.end_pos),
-                                  im.TextNode('="', pos=node.end_pos),
-                                  im.InterpolateNode(
-                                      value='__piglet_attr_v', pos=node.end_pos),
-                                  im.TextNode('"', pos=node.end_pos)])
+        for_ = im.ForNode(
+            each=("__piglet_attr_k, __piglet_attr_v " "in __piglet_attrs.items()"),
+            pos=node.end_pos,
+            children=[],
+        )
+        if_ = im.IfNode(
+            test="__piglet_attr_v is not None",
+            pos=node.end_pos,
+            children=[
+                im.TextNode(" ", pos=node.end_pos),
+                im.InterpolateNode(value="__piglet_attr_k", pos=node.end_pos),
+                im.TextNode('="', pos=node.end_pos),
+                im.InterpolateNode(value="__piglet_attr_v", pos=node.end_pos),
+                im.TextNode('"', pos=node.end_pos),
+            ],
+        )
         for_.children.append(if_)
         wn.append(for_)
     wn.append(im.TextNode(content=node.end, pos=node.end_pos))
@@ -797,60 +873,75 @@ def _compile_attr(a, container):
     """
     Compile a single parse.Attribute
     """
-    intro_text = im.TextNode.factory('{0.name}{0.space1}='
-                                        '{0.space2}{0.quote}'.format(a),
-                                     pos=a.pos)
-    outro_text = im.TextNode.factory('{0.quote}{0.space3}'.format(a),
-                                     pos=a.value_pos.advance(a.value))
+    intro_text = im.TextNode.factory(
+        "{0.name}{0.space1}={0.space2}{0.quote}".format(a), pos=a.pos
+    )
+    outro_text = im.TextNode.factory(
+        "{0.quote}{0.space3}".format(a), pos=a.value_pos.advance(a.value)
+    )
 
     items = interpolate.parse_interpolations(a.value)
 
-    if a.name in HTML_EMPTY_ATTRS and \
-       len(items) == 1 and \
-       isinstance(items[0], interpolate.Interpolation):
+    if (
+        a.name in HTML_EMPTY_ATTRS
+        and len(items) == 1
+        and isinstance(items[0], interpolate.Interpolation)
+    ):
         interp = items[0]
         container.append(
             im.WithNode(
-                vars=[('__piglet_tmp', interp.value)],
+                vars=[("__piglet_tmp", interp.value)],
                 pos=a.pos,
                 children=[
-                    im.IfNode(test='__piglet_tmp is not None',
-                              pos=a.pos,
-                              children=[
-                                  intro_text,
-                                  im.InterpolateNode(value='__piglet_tmp',
-                                                     pos=a.value_pos),
-                                  outro_text
-                              ])
-                ])
+                    im.IfNode(
+                        test="__piglet_tmp is not None",
+                        pos=a.pos,
+                        children=[
+                            intro_text,
+                            im.InterpolateNode(value="__piglet_tmp", pos=a.value_pos),
+                            outro_text,
+                        ],
+                    )
+                ],
+            )
         )
     else:
         container.append(intro_text)
         pos = a.value_pos
         for i in items:
-            if isinstance(i, (str, ustr)):
+            if isinstance(i, str):
                 container.append(im.TextNode.factory(i, pos))
                 pos = pos.advance(i)
             else:
-                container.append(im.InterpolateNode(value=unescape(i.value),
-                                                    pos=pos))
+                container.append(im.InterpolateNode(value=unescape(i.value), pos=pos))
                 pos = pos.advance(i.source)
         outro_text.pos = pos
         container.append(outro_text)
 
 
-def _compile_close_tag(node, strip_condition=None):
-    if isinstance(node, Fragment) or not node.close_tag:
-        return im.TextNode('', pos=node.close_tag_pos)
+def _compile_close_tag(node, tagname_expr=None, strip_condition=None):
+    if not node.close_tag or (isinstance(node, Fragment) and not tagname_expr):
+        return im.TextNode("", pos=node.close_tag_pos)
 
-    closetag = im.TextNode(node.close_tag, pos=node.close_tag_pos)
+    if tagname_expr:
+        closetag = im.ContainerNode(
+            children=[
+                im.TextNode.factory("</", pos=node.close_tag_pos),
+                im.InterpolateNode(value=tagname_expr, pos=node.close_tag_pos),
+                im.TextNode.factory(">", pos=node.close_tag_pos),
+            ]
+        )
+    else:
+        closetag = im.TextNode(node.close_tag, pos=node.close_tag_pos)
     if strip_condition:
-        if strip_condition in {'True', '1'}:
+        if strip_condition in {"True", "1"}:
             return im.NullNode(children=[closetag])
         else:
-            return im.IfNode(test='not ({})'.format(strip_condition),
-                             children=[closetag],
-                             pos=node.close_tag_pos)
+            return im.IfNode(
+                test="not ({})".format(strip_condition),
+                children=[closetag],
+                pos=node.close_tag_pos,
+            )
     return closetag
 
 
@@ -866,7 +957,8 @@ def append_directive_nodes(srcnode, directives, container):
     working_node = container
     for directive, data, pos in directives:
         if directive.factory:
-            child = directive.factory(**data)
+            kwargs = {k.replace("-", "_"): v for k, v in data.items()}
+            child = directive.factory(**kwargs)
             child.pos = pos
             for _, item, _ in im.walk_tree_pre(child):
                 item.pos = pos
@@ -894,12 +986,17 @@ def _compile_element_with_directives(orig, directives, remaining):
         extras = d.get_extra_attrs(dd, remaining)
         if extras:
             extra_attrs.append(extras)
-        strip_condition = (d.get_strip_condition(dd, remaining) or
-                           strip_condition)
+        strip_condition = d.get_strip_condition(dd, remaining) or strip_condition
+        tagname_expr = d.get_tagname_expr(dd, remaining)
 
-    working_node.append(_compile_open_tag(remaining,
-                                          strip_condition=strip_condition,
-                                          extra_attrs=extra_attrs))
+    working_node.append(
+        _compile_open_tag(
+            remaining,
+            tagname_expr=tagname_expr,
+            strip_condition=strip_condition,
+            extra_attrs=extra_attrs,
+        )
+    )
 
     # Add inner directives
     inner_working_node = append_directive_nodes(orig, inner_directives, working_node)
@@ -910,7 +1007,10 @@ def _compile_element_with_directives(orig, directives, remaining):
 
     inner_working_node.extend(map(_compile_node, remaining.children))
 
-    working_node.append(_compile_close_tag(remaining,
-                                           strip_condition=strip_condition))
+    working_node.append(
+        _compile_close_tag(
+            remaining, tagname_expr=tagname_expr, strip_condition=strip_condition
+        )
+    )
 
     return container
