@@ -35,7 +35,7 @@ class Interpolation(object):
 
 
 parser_ns = locals()
-interp_grammar = resource_string(__name__, 'grammar_interpolate.txt').decode('UTF-8')
+interp_grammar = resource_string(__name__, "grammar_interpolate.txt").decode("UTF-8")
 interp_parser = parsley.makeGrammar(interp_grammar, parser_ns)
 
 
