@@ -22,8 +22,6 @@ from piglet import compile as pigletcompile
 from piglet.exceptions import PigletParseError
 from piglet.runtime import data as rtdata, munge_exception_messages
 
-ustr = type(u'')
-
 
 class BaseTemplate:
 
@@ -33,13 +31,15 @@ class BaseTemplate:
     codeob = None
     intermediate = None
 
-    def __init__(self,
-                 source=None,
-                 filename='<string>',
-                 loader=None,
-                 translations_factory=None,
-                 save_pysrc_to=None,
-                 codeob=None):
+    def __init__(
+        self,
+        source=None,
+        filename="<string>",
+        loader=None,
+        translations_factory=None,
+        save_pysrc_to=None,
+        codeob=None,
+    ):
         self.filename = filename
         self.loader = loader
         if translations_factory:
@@ -47,9 +47,9 @@ class BaseTemplate:
         try:
             if codeob is None:
                 self.intermediate = self.compile_intermediate(source)
-                codeob = pigletcompile.compile_to_codeob(self.intermediate,
-                                                         filename,
-                                                         saveas=save_pysrc_to)
+                codeob = pigletcompile.compile_to_codeob(
+                    self.intermediate, filename, saveas=save_pysrc_to
+                )
             self.codeob = codeob
             self.template_module = self._get_module()
         except PigletParseError as e:
@@ -59,32 +59,39 @@ class BaseTemplate:
         self.root_fn = self.template_module.__piglet_root__
 
     @classmethod
-    def from_pysrc(cls,
-                    pysrc,
-                    pysrc_filename='<string>',
-                    filename='<string>',
-                    loader=None,
-                    translations_factory=None):
+    def from_pysrc(
+        cls,
+        pysrc,
+        pysrc_filename="<string>",
+        filename="<string>",
+        loader=None,
+        translations_factory=None,
+    ):
 
-        return cls(filename=filename,
-                   codeob=compile(pysrc, pysrc_filename, 'exec'),
-                   loader=loader,
-                   translations_factory=translations_factory)
+        return cls(
+            filename=filename,
+            codeob=compile(pysrc, pysrc_filename, "exec"),
+            loader=loader,
+            translations_factory=translations_factory,
+        )
 
     def __repr__(self):
         return "<{} {!r}>".format(type(self).__name__, self.filename)
 
     def __call__(self, context, *args, **kwargs):
-        if not hasattr(rtdata, 'context'):
+        if not hasattr(rtdata, "context"):
             rtdata.context = []
         rtdata.context.append(context)
         rtdata.exception_locations = []
         translations = self.translations_factory()
-        context.update({'_': translations.gettext,
-                        'gettext': translations.gettext,
-                        'ngettext': translations.ngettext})
-        content = munge_exception_messages(self.root_fn(*args, **kwargs),
-                                           context)
+        context.update(
+            {
+                "_": translations.gettext,
+                "gettext": translations.gettext,
+                "ngettext": translations.ngettext,
+            }
+        )
+        content = munge_exception_messages(self.root_fn(*args, **kwargs), context)
         try:
             for s in content:
                 yield s
@@ -93,28 +100,27 @@ class BaseTemplate:
 
     def __getstate__(self):
         d = self.__dict__.copy()
-        del d['template_module']
-        del d['root_fn']
-        d['codeob'] = marshal.dumps(self.codeob)
+        del d["template_module"]
+        del d["root_fn"]
+        d["codeob"] = marshal.dumps(self.codeob)
         return d
 
     def __setstate__(self, state):
-        state['codeob'] = marshal.loads(state['codeob'])
+        state["codeob"] = marshal.loads(state["codeob"])
         self.__dict__.update(state)
-        self.__dict__['template_module'] = self._get_module()
+        self.__dict__["template_module"] = self._get_module()
         self.root_fn = self.template_module.__piglet_root__
 
     def _get_module(self):
         return pigletcompile.compile_to_module(
-            self.codeob,
-            self.filename,
-            bootstrap={'__piglet_template': self})
+            self.codeob, self.filename, bootstrap={"__piglet_template": self}
+        )
 
     def compile_intermediate(self):
         raise NotImplementedError()
 
     def render(self, context, *args, **kwargs):
-        return ''.join(map(ustr, self(context, *args, **kwargs)))
+        return "".join(map(str, self(context, *args, **kwargs)))
 
     def get_pysrc(self):
         """
@@ -131,8 +137,8 @@ class BaseTemplate:
         Expose template functions as attributes, eg:
 
             >>> t = Template('<py:def function="greeting">Hello!</py:def>')
-            >>> print(t.greeting())
-            'Hello!'
+            >>> print(t.greeting)
+            <function greeting at ...>
         """
         return getattr(self.template_module, name)
 
